# -*- coding: utf-8 -*-

import threading
import skypebotconfig
import feedparser
from time import sleep

ringbuf_size = 200

class RSSPoller(threading.Thread):
    def __init__(self, feedurl, skype, chat, title, polltime = 60):
        threading.Thread.__init__(self)
        self.feedurl = feedurl
        self.skype = skype
        self.chat = chat
        self.title = title
        self.latest = []
        self.current_item = 0
        self.polltime = polltime
        self.running = True
        
    def _send_entry_to_chat(self, feed, entry):
        self.skype.Chat(self.chat).SendMessage((u'%s: %s < %s >' % 
             (self.title, entry.title, entry.link)))

    def run(self):
        while self.running:
            try:
                d = feedparser.parse(self.feedurl)

                first_run = True
                if len(self.latest) > 0:
                   first_run = False

                new_entries = []
                
                for entry in d.entries:
                    if entry.link not in self.latest:
                        new_entries.append(entry)
                      
                        if len(self.latest) == ringbuf_size:
                            self.latest[self.current_item] = entry.link
                        else:
                            self.latest.append(entry.link)
                        self.current_item = (self.current_item + 1) % ringbuf_size

                if not first_run:
                    for entry in new_entries[::-1]:
                        self._send_entry_to_chat(d.feed, entry)
           
            except:
                import sys
                print "RSSPoller for", self.feedurl, "encountered an error:", sys.exc_info()
            
            sleep(self.polltime)
                
def run_pollers(skype):
    for feedurl, chat, title, delay in skypebotconfig.RSS_FEEDS:
        RSSPoller(feedurl, skype, chat, title, delay).start()
