#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Python / Skype4Py chat bot för Piratpartiet

Tested with  Skype4Py version 0.9.28.5 and Skype verson 3.8.0.96
"""

import Skype4Py
import re
import httplib
import base64
import urllib
import skypebotconfig
from rsspoller import run_pollers
from tinyurl import tinyfy_string
from wiki import Wiki
import simplejson as json
import tweepy

command_re = re.compile(' *ppbot: *(\w*) *(.*)$')
forwardmessage_re = re.compile('(\w+): *(.*)$', re.UNICODE)
forbidden_channelids = ("http", "https", "ftp")
http_re = re.compile(r"(https?://[^ ]+)")

wiki = Wiki()

def log_message(Message):
    """
    Inject message to logging skype channel
    """
    logchannel = None

    for logchannelpair in skypebotconfig.LOGCHANNELS:
      if Message.Chat.Name == logchannelpair[0]:
        logchannel = logchannelpair[1]
        break

    if logchannel == None:
      return

    skype.Chat(logchannel).SendMessage(
        Message.FromDisplayName + ': ' + Message.Body)

def log_twitter(Message):
    """
    Put message on twitter
    """


    fb_mess = twit = Message.Body[3:].strip()

    full_twit = twit

    if twit.find("#svpol") == -1:
      twit += " #svpol"

    if len(twit) > 140:
        twit = tinyfy_string(twit)

    if len(twit) > 140:
        Message.Chat.SendMessage(u"För lång tweet (%d tkn): %s" % (len(twit) - 140, twit))
        return;

    twitencoded = twit.encode("utf-8")
   
    if skypebotconfig.twitter_consumer_key != "":

       status = None
       try:
          status = twitter.update_status(twitencoded)
       except tweepy.TweepError as err:
          Message.Chat.SendMessage("Twitter error: %d: %s" % (err.response.status, err.reason))
          return;


       skype.Chat(skypebotconfig.twitterlogchannel).SendMessage("Twitter:   " +
          full_twit + "   (twittrade OK, %d tecken, http://twitter.com/%s/status/%d )" \
          % (len(twit), status.author.screen_name, status.id))
    
    if skypebotconfig.fb_access_token != "":
 
       params = {}   
       m = http_re.search(fb_mess)
       if m:
         params['link'] = m.group()

       params['message'] = http_re.sub("", fb_mess, 1).encode('utf-8')

       p = urllib.urlencode(params)

       f = urllib.urlopen("https://graph.facebook.com/me/feed?access_token=%s" % skypebotconfig.fb_access_token, p)
       
       if f.getcode() != 200:
          Message.Chat.SendMessage("Facebook error: %d: %s" % (f.getcode(), f.read()))


def on_attach(status):
    """
    Fired on attachment status change. Here used to re-attach
    this script to Skype in case attachment is lost. Just in case.
    """
    print 'API attachment status: ' + \
        skype.Convert.AttachmentStatusToText(status)
    if status == Skype4Py.apiAttachAvailable:
        skype.Attach()
    if status == Skype4Py.apiAttachSuccess:
        print '**************************'

def on_message_status(Message, Status):
    """
    Fired on chat message status change.
    Statuses can be: 'UNKNOWN' 'SENDING' 'SENT' 'RECEIVED' 'READ'
    """
    if Status not in ('RECEIVED', 'SENT'):
        return

    if Status == 'SENT' and not skypebotconfig.debug:
        return

    print "%s, in %s,%s: %s" % (Message.FromDisplayName, Message.Chat.Name,
                                Message.Chat.Topic, Message.Body)
  

    loggable = False

    if Message.Body.startswith('-'):
        return

    if Message.Body == 'm':
        Message.Chat.SendMessage(Message.FromDisplayName + u" säger +1 emma")
        return
   
    if re.match("^[Hh]ej +ppbot[!]?$", Message.Body):
        Message.Chat.SendMessage(u"Hej " + Message.FromDisplayName + u"! Hur är det med dig idag?")
        return

    if Message.Body.startswith("!") and len(Message.Body) > 2:
        loggable = True

    if Message.Body.startswith('+++ ') and Message.Chat.Name in skypebotconfig.twitterchannels:
        log_twitter(Message)
        return

    if Message.Body.startswith('wiki:'):
        Message.Chat.SendMessage(wiki.handle_skype_message(
                Message.FromDisplayName, Message.Body[5:]))

    if Message.Body.find("ppbot:") == 0:
        exec_command(Message)
        return

    if forwardmessage_re.match(Message.Body):
        forward_message(Message)

    if Message.Body.find("http://") >= 0:
        loggable = True

    if Message.Body.find("https://") >= 0:
        loggable = True

    if loggable:
        log_message(Message)


def exec_command(Message):
    """
    Chat command dispatcher
    """
    matches = command_re.match(Message.Body)
    command = matches.group(1)
    args = matches.group(2)
    print 'Command: ' + command
    print 'Args: ' + args
    if command == 'members':
        print_members(Message.Chat)
    elif command == 'list':
        print_cluster(skype.CreateChatWith(Message.Sender.Handle))
    elif command == 'getchatid':
        Message.Chat.SendMessage("Chatt-ID: " + Message.Chat.Name)
    elif command == 'geturl':
        Message.Chat.SendMessage("URL: " + Message.Chat.Blob)
    elif command == 'inviteme':
        invite_self(Message, args)
    elif command == 'help':
        print_help(skype.CreateChatWith(Message.Sender.Handle))



def invite_self(Message, args):
    """
    Invite self to other chat
    """
    channelid = args.strip()

    channelskypeid = None

    for channeldesc in skypebotconfig.CHAT_CLUSTER:
       if channelid == channeldesc[0]:
          channelskypeid = channeldesc[1]
          if not channeldesc[3]:
             Message.Chat.SendMessage(u"Kanalen är inte öppen för egenanmälan: " + channelid)
             return
          break
    
    if channelskypeid == None:
       Message.Chat.SendMessage(u"Okänd kanal: " + channelid)
       return

    skype.Chat(channelskypeid).AddMembers(Message.Sender)
    skype.Chat(channelskypeid).SendMessage(u"Välkommen " + Message.FromDisplayName + "!")


def forward_message(Message):
    """
    Forward message
    """
    matches = forwardmessage_re.match(Message.Body)
    channelid = matches.group(1)
    message = matches.group(2)

    if channelid in forbidden_channelids:
      return

    channelskypeid = None
    sourcechannelname = None
    sourcechannelid = None


    for channeldesc in skypebotconfig.CHAT_CLUSTER:
       if Message.Chat.Name == channeldesc[1]:
          sourcechannelname = channeldesc[2]
          sourcechannelid   = channeldesc[0]
          break

    if sourcechannelname == None:
       sourcechannelname = Message.Chat.Topic

    if sourcechannelid == None:
       fullmessage = u"Från " + Message.FromDisplayName + " (" + Message.FromHandle + "): " + message
    else: 
       fullmessage = u"Från " + Message.FromDisplayName + " i " + sourcechannelname + " (" + sourcechannelid + "): " + message

    if channelid == u"halärm":
      for channeldesc in skypebotconfig.CHAT_CLUSTER:
         skype.Chat(channeldesc[1]).SendMessage(fullmessage)

    for channeldesc in skypebotconfig.CHAT_CLUSTER:
       if channelid == channeldesc[0]:
          channelskypeid = channeldesc[1]
          break
    
    if channelskypeid != None:
      print 'Forward "' + message + '" to ' + channelid
      skype.Chat(channelskypeid).SendMessage(fullmessage)
    else: 
       # Send to contact 'channelid'
       for friend in skype.Friends:
         if friend.Handle == channelid:
            print 'Forward "' + message + '" to user ' + channelid 
            skype.CreateChatWith(channelid).SendMessage(fullmessage)
       

def print_help(Chat):
    """
    Print help message to chat
    """

    Chat.SendMessage(u"""ppbot kommandon:
ppbot: members\t--- skriver ut antal medlemmar
ppbot: help\t\t--- detta meddelande
ppbot: list\t\t--- listar kanaler som går att skicka meddelanden till
ppbot: getchatid\t\t--- ger ett ID för denna chatt som kan användas för att lägga till en chatt till klustret.
ppbot: inviteme [chatid]\t--- bjuder in dig till kanalen [chatid]. Du måste ha pp-bot i din kontaktlista först!

Om meddelanden:
[chatid]: [meddelande]\t--- skickar ett meddelande till chatten [chatid]

Om twitter:
+++ meddelande\t--- skickar ett twittermeddelande
+++ @user meddelande \t--- skickar ett reply till "user"

Om loggkanalen:
Allt med länkar i loggas. Dessutom:
! meddelande\t--- skickar ett meddelande till loggkanalen
- bla http://xyz\t--- skickar INTE länken till loggkanalen

Om wiki:
wiki: [meddelande] \t--- Skapar en wikisida med texten [meddelande] och returnerar URL:en
""")


def print_members(Chat):
    """
    Print current member count to Chat
    """
    conn = httplib.HTTPConnection("lekstuga.piratpartiet.se")
    conn.request("GET", "/membersfeed")
    resp = conn.getresponse()
    if(resp.status == 200):
        data = resp.read()
        Chat.SendMessage('PP Members: ' + data)
    else:
        Chat.SendMessage('Error: %d' + resp.status)
    conn.close()


def print_cluster(Chat):
    """
    Print current cluster to Chat
    """
    message = u"Nuvarande skypekluster ((*) = invite only):\n"

    for channeldesc in skypebotconfig.CHAT_CLUSTER:
       io = ""
       if not channeldesc[3]:
          io = " (*)"
       message += channeldesc[0] + ": " + channeldesc[2] + io + "\n"

    Chat.SendMessage(message)

auth = tweepy.OAuthHandler(skypebotconfig.twitter_consumer_key, skypebotconfig.twitter_consumer_secret)
auth.set_access_token(skypebotconfig.twitter_access_token, skypebotconfig.twitter_access_secret)

twitter = tweepy.API(auth)


skype = Skype4Py.Skype()
skype.OnAttachmentStatus = on_attach
skype.OnMessageStatus = on_message_status
print '**************************'
print 'Connecting to Skype..'
skype.Attach()
run_pollers(skype)

cmd = ''
while not cmd == 'exit':
    cmd = raw_input('')

import os, signal
os.kill(os.getpid(), signal.SIGTERM)
